import chalk from 'chalk';
import * as fs from 'fs-jetpack';
import * as os from 'os';
import * as path from 'path';
import * as stackTraceParser from 'stacktrace-parser';
import { highlightTS } from './highlight';
const schemaRegex = /(\S+(objectType|inputObjectType|interfaceType|unionType|enumType|queryType|mutationType|subscriptionType|extendType|scalarType|importType|)\()/;
export const printStack = ({ callsite }) => {
    let fileLineNumber = null;
    let filePreview = null;
    let filePath = null;
    let filePathRelToHomeDir = null;
    let methodName = null;
    // @ts-ignore
    if (callsite && typeof window === 'undefined') {
        const stack = stackTraceParser.parse(callsite);
        // TODO: more resilient logic to find the right trace
        const trace = stack.find((t) => t.file &&
            !t.file.includes('node_modules/nexus') &&
            !t.file.includes('node_modules/@nexus/schema') &&
            !t.file.includes('node_modules/graphql'));
        if (process.env.NEXUS_STAGE === 'dev' &&
            trace &&
            trace.file &&
            trace.lineNumber &&
            trace.column &&
            !trace.file.startsWith('internal/')) {
            const lineNumber = trace.lineNumber;
            const projectRoot = getProjectRoot();
            const tracePathRelToProjectRoot = projectRoot ? path.relative(projectRoot, trace.file) : trace.file;
            const tracePathRelToHomeDir = trace.file.replace(os.homedir(), '~');
            fileLineNumber = callsite
                ? `${chalk.underline(`${tracePathRelToProjectRoot}:${lineNumber}:${trace.column}`)}`
                : '';
            if (fs.exists(trace.file)) {
                const fileContent = fs.read(trace.file);
                const splitFile = fileContent.split('\n');
                const start = Math.max(0, lineNumber - 3);
                const end = Math.min(lineNumber + 3, splitFile.length - 1);
                const lines = splitFile.slice(start, end);
                const theLine = lines[2];
                const match = theLine.match(schemaRegex);
                if (match) {
                    methodName = `${match[1]})`;
                }
                const highlightedLines = highlightTS(lines.join('\n')).split('\n');
                filePreview = highlightedLines
                    .map((l, i) => chalk.grey(renderN(i + start + 1, lineNumber + start + 1) + ' ') + chalk.reset() + l)
                    .map((l, i, _arr) => i === 2
                    ? `${chalk.red.bold('→')} ${l} ${chalk.dim(`${tracePathRelToHomeDir}:${lineNumber}:${trace.column}`)}`
                    : chalk.dim('  ' + l))
                    .join('\n');
                filePath = trace.file;
                filePathRelToHomeDir = tracePathRelToHomeDir;
            }
        }
    }
    return {
        preview: filePreview ? `${filePreview}${chalk.reset()}` : null,
        methodName,
        file: {
            path: filePath,
            pathLineNumber: fileLineNumber,
            pathRelToHomeDir: filePathRelToHomeDir,
        },
    };
};
function renderN(n, max) {
    const wantedLetters = String(max).length;
    const hasLetters = String(n).length;
    if (hasLetters >= wantedLetters) {
        return String(n);
    }
    return String(' '.repeat(wantedLetters - hasLetters) + n);
}
/**
 * Stack overflow reference: https://stackoverflow.com/a/43960876
 */
export function getProjectRoot() {
    var _a, _b;
    return (_b = (_a = process === null || process === void 0 ? void 0 : process.mainModule) === null || _a === void 0 ? void 0 : _a.paths[0].split('node_modules')[0].slice(0, -1)) !== null && _b !== void 0 ? _b : null;
}
//# sourceMappingURL=print-stack.js.map